package de.intevation.flys.artifacts.model.fixings;

import org.apache.log4j.Logger;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;

import de.intevation.flys.utils.KMIndex;

import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.FixationArtifactAccess;

import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.DataFacet;
import de.intevation.flys.artifacts.model.CalculationResult;

import de.intevation.flys.artifacts.states.DefaultState.ComputeType;


/**
 * Facet to show W values for Q values at km for a date.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class FixAnalysisEventsFacet
extends      DataFacet
implements   FacetTypes {

    /** House logger. */
    private static Logger logger = Logger.getLogger(FixAnalysisEventsFacet.class);

    /** Trivial Constructor. */
    public FixAnalysisEventsFacet() {
    }


    /**
     * @param name
     */
    public FixAnalysisEventsFacet(int index, String name, String description) {
        super(index,
             name,
             description,
             ComputeType.ADVANCE,
             null,
             null);
    }


    /**
     * Returns the data this facet requires.
     *
     * @param artifact the owner artifact.
     * @param context  the CallContext (ignored).
     *
     * @return the data.
     */
    @Override
    public Object getData(Artifact artifact, CallContext context) {
        logger.debug("FixAnalysisEventsFacet.getData");

        if (artifact instanceof FLYSArtifact) {
            FLYSArtifact flys = (FLYSArtifact)artifact;
            FixationArtifactAccess access = new FixationArtifactAccess(flys);

            CalculationResult res =
                (CalculationResult) flys.compute(context,
                                                 ComputeType.ADVANCE,
                                                 false);

            FixResult result = (FixResult) res.getData();
            Double currentKm =
                ((Double)context.getContextValue("currentKm")).doubleValue();

            KMIndex<AnalysisPeriod []> kmPeriods = result.getAnalysisPeriods();
            KMIndex.Entry<AnalysisPeriod []> kmPeriodsEntry =
                kmPeriods.binarySearch(currentKm);

            if(kmPeriodsEntry == null) {
                return null;
            }

            AnalysisPeriod[] periods = kmPeriodsEntry.getValue();
            if (periods == null) {
                return null;
            }
            QWD[] qwdData = periods[index].getQWDs();
            return qwdData;
        }
        else {
            logger.debug("Not an instance of FixationArtifact.");
            return null;
        }
    }


    /**
     * Create a deep copy of this Facet.
     * @return a deep copy.
     */
    @Override
    public FixAnalysisEventsFacet deepCopy() {
        FixAnalysisEventsFacet copy = new FixAnalysisEventsFacet();
        copy.set(this);
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
